
/*
 * The Real SoundTracker - GUI (menu bar)
 *
 * Copyright (C) 1999-2019 Michael Krause
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <config.h>

#include <gdk/gdkkeysyms.h>
#include <glib/gi18n.h>
#include <string.h>

#include "audioconfig.h"
#include "cheat-sheet.h"
#include "file-operations.h"
#include "gui-settings.h"
#include "gui-subs.h"
#include "gui.h"
#include "instrument-editor.h"
#include "keys.h"
#include "main.h"
#include "menubar.h"
#include "midi-settings.h"
#include "module-info.h"
#include "preferences.h"
#include "sample-editor.h"
#include "scope-group.h"
#include "st-subs.h"
#include "tips-dialog.h"
#include "track-editor.h"
#include "tracker-settings.h"
#include "transposition.h"

extern GtkBuilder *gui_builder;

static GtkWidget* mark_mode;

static gboolean mark_mode_toggle_ignore = FALSE;

extern ScopeGroup* scopegroup;

void about_dialog(void)
{
    const gchar* const authors[] = {
        "Michael Krause <rawstyle@soundtracker.org>",
        "Yury Aliaev <mutab0r@rambler.ru>",
        "Jon Forsberg <zzed@cyberdude.com>",
        "Arthibus Gissehel <gissehel@vachefolle.com>",
        "Stefan Hager <b.hager@eduhi.at>",
        "Maik Naeher <m_naeher@informatik.uni-kl.de>",
        "Darin Ohashi <dmohashi@algonquin.uwaterloo.ca>",
        "Michael Schwendt <sidplay@geocities.com>",
        "Erik Thiele <erikyyy@studbox.uni-stuttgart.de>",
        "Kai Vehmanen <kai.vehmanen@wakkanet.fi>",
        "Yuuki Ninomiya <gm@smn.enjoy.ne.jp>",
        "Martin Andersson <t3tr1s@yahoo.com>",
        "Conrad Parker <conradp@cse.unsw.edu.au>",
        "Tomasz Maka <pasp@ll.pl>",
        "Nicolas Leveille <nicolas.leveille@free.fr>",
        "Jason Nunn <jsno@downunder.net.au>",
        "Luc Tanguay <luc.tanguay@bell.ca>",
        "Tijs van Bakel <smoke@casema.net>",
        "Olivier Glorieux <sglorzlinux@netcourrier.com>",
        "Rikard Bosnjakovic <bos@hack.org>",
        "Fabian Giesen <fg@farb-rausch.de>",
        "Felix Bohmann <fb@farb-rausch.de>",
        "Aki Kemppainen <kemppaia@assari.cc.tut.fi>",
        "Mardy <mardy@users.sourceforge.net>",
        "Thomas Uwe Gruettmueller <sloyment@gmx.net>",
        "Wilbern Cobb <wcobb@openbsd.org>",
        "Erik de Castro Lopo <erikd-100@mega-nerd.com>",
        "Anthony Van Groningen <avan@uwm.edu>",
        "Henri Jylkk\303\244 <hj@bsdmail.org>",
        "Frank Knappe <knappe@tu-harburg.de>",
        "Mathias Meyer <mathmeye@users.sourceforge.net>",
        "David Binderman <dcb314@hotmail.com>",
        "Jaan Pullerits <jaan@surnuaed.ee>",
        "Olivier Guilyardi <ml@xung.org>",
        "Markku Reunanen <marq@iki.fi>",
        "Miroslav Shatlev <coding@shaltev.org>",
        "Frank Haferkorn <WillyFoobar@gmx.de>",
        "St\303\251phane K. <stephanek@brutele.be>",
        "Matthew Berardi <metaquanta@users.sourceforge.net>",
        "Bernardo Innocenti <bernie@codewiz.org>",
        NULL };
    const gchar translators[] = "Colin Marquardt <colin.marquardt@gmx.de>\nThomas R. Koll <tomk32@tomk32.de>\n"
        "Atsushi Yamagata <yamagata@plathome.co.jp>\nYuuki Ninomiya <gm@smn.enjoy.ne.jp>\n"
        "Junichi Uekawa <dancer@netfort.gr.jp>\nYuri Bongiorno <yurix@tiscalinet.it>\n"
        "Zbigniew Chyla <chyla@alice.ci.pwr.wroc.pl>\n"
        "German Jose Gomez Garcia <german@pinon.ccu.uniovi.es>\n"
        "Vicente E. Llorens <vllorens@mundofree.com>\nMichael Shigorin <mike@altlinux.ru>\n"
        "Yury Aliaev <mutab0r@rambler.ru>\nMichel Robitaille <robitail@IRO.UMontreal.CA>\n"
        "europeen <karlkawada@wanadoo.fr>\nMatej Erman <matej.erman@guest.arnes.si>\n"
        "Xos\303\251 Anxo Pereira Torreiro <pereira@iponet.es>\nChristian Rose <menthos@menthos.com>\n"
        "Denis Lackovic <delacko@fly.srk.fer.hr>\nKeld Simonsen <keld@dkuug.dk>\n"
        "Petter Johan Olsen <petter.olsen@cc.uit.no>\nAndrej Kacian <andrej@kacian.sk>\n"
        "Steven Michael Murphy <murf@e-tools.com>\nAysun Kiran <aysunkir@yahoo.com>\n"
        "Clytie Siddall <clytie@riverland.net.au>";

    gchar *license = g_strdup_printf(_("%s is free software: you can redistribute it and/or modify it under the terms "
        "of the GNU General Public License as published by the Free Software Foundation; either version 2 "
        "of the License, or (at your option) any later version.\n\n"
        "%s is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the "
        "implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General "
        "Public License for more details.\n\n"
        "You should have received a copy of the GNU General Public License along with %s"
        ". If not, see: http://www.gnu.org/licenses/"), PACKAGE_NAME, PACKAGE_NAME, PACKAGE_NAME);

    gtk_show_about_dialog(GTK_WINDOW(mainwindow), "program_name", PACKAGE_NAME, "version", VERSION,
        "authors", authors, "license", license, "wrap-license", TRUE, "copyright",
        _("\302\251 1998-2019 Michael Krause\n\302\251 2020 Yury Aliaev\n"
        "\n\nIncludes OpenCP player from Niklas Beisert and Tammo Hinrichs."),
        "translator-credits", translators,
        NULL);
    g_free(license);
}

static void
menubar_clear(gboolean all)
{
    if (all) {
        gui_free_xm();
        gui_new_xm();
    } else {
        gui_play_stop();
        st_clean_song(xm);
        gui_init_xm(1, TRUE, FALSE);
    }
}

void menubar_clear_clicked(gpointer b)
{
    if (xm_get_modified()) {
        if (gui_ok_cancel_modal((mainwindow),
                _("Are you sure you want to do this?\nAll changes will be lost!")))
            menubar_clear(GPOINTER_TO_INT(b));
    } else {
        menubar_clear(GPOINTER_TO_INT(b));
    }
}

void menubar_backing_store_toggled(GtkCheckMenuItem* widget)
{
    gui_settings.gui_use_backing_store = gtk_check_menu_item_get_active(widget);
    tracker_set_backing_store(tracker, gui_settings.gui_use_backing_store);
}

void menubar_scopes_toggled(GtkCheckMenuItem* widget)
{
    gui_settings.gui_display_scopes = gtk_check_menu_item_get_active(widget);
    scope_group_enable_scopes(scopegroup, gui_settings.gui_display_scopes);
}

void menubar_splash_toggled(GtkCheckMenuItem* widget)
{
    gui_settings.gui_disable_splash = gtk_check_menu_item_get_active(widget);
}

void menubar_mark_mode_toggled(GtkCheckMenuItem* widget)
{
    if (!mark_mode_toggle_ignore) {
        gboolean state = gtk_check_menu_item_get_active(widget);
        tracker_mark_selection(tracker, state);
    }
}

void menubar_save_settings_on_exit_toggled(GtkCheckMenuItem* widget)
{
    gui_settings.save_settings_on_exit = gtk_check_menu_item_get_active(widget);
}

void menubar_save_settings_now(void)
{
    gui_settings_save_config();
    keys_save_config();
    audioconfig_save_config();
    trackersettings_write_settings();
#if defined(DRIVER_ALSA_MIDI)
    midi_save_config();
#endif
    prefs_save();
}

void menubar_handle_cutcopypaste(gpointer a)
{
    static const gchar* signals[] = { "cut-clipboard", "copy-clipboard", "paste-clipboard" };

    Tracker* t = tracker;

    STInstrument* curins = &xm->instruments[gui_get_current_instrument() - 1];
    GtkWidget* focus_widget = GTK_WINDOW(mainwindow)->focus_widget;
    gint i = GPOINTER_TO_INT(a);

    if (GTK_IS_ENTRY(focus_widget)) {
        g_signal_emit_by_name(focus_widget, signals[i], NULL);
        return;
    }

    switch (i) {
    case 0: //Cut
        switch (notebook_current_page) {
        case NOTEBOOK_PAGE_TRACKER:
            if (GUI_EDITING)
                track_editor_cut_selection(NULL, t);
            break;
        case NOTEBOOK_PAGE_INSTRUMENT_EDITOR:
        case NOTEBOOK_PAGE_MODULE_INFO:
            instrument_editor_cut_instrument(curins);
            gui_xm_set_modified(1);
            instrument_editor_update(TRUE);
            sample_editor_update();
            break;
        case NOTEBOOK_PAGE_SAMPLE_EDITOR:
            sample_editor_copy_cut_common(TRUE, TRUE);
            gui_xm_set_modified(1);
            break;
        }
        break;
    case 1: //Copy
        switch (notebook_current_page) {
        case NOTEBOOK_PAGE_TRACKER:
            track_editor_copy_selection(NULL, t);
            break;
        case NOTEBOOK_PAGE_INSTRUMENT_EDITOR:
        case NOTEBOOK_PAGE_MODULE_INFO:
            instrument_editor_copy_instrument(curins);
            break;
        case NOTEBOOK_PAGE_SAMPLE_EDITOR:
            sample_editor_copy_cut_common(TRUE, FALSE);
            break;
        }
        break;
    case 2: //Paste
        switch (notebook_current_page) {
        case NOTEBOOK_PAGE_TRACKER:
            if (GUI_EDITING)
                track_editor_paste_selection(NULL, t);
            break;
        case NOTEBOOK_PAGE_INSTRUMENT_EDITOR:
        case NOTEBOOK_PAGE_MODULE_INFO:
            instrument_editor_paste_instrument(curins);
            gui_xm_set_modified(1);
            instrument_editor_update(TRUE);
            sample_editor_update();
            break;
        case NOTEBOOK_PAGE_SAMPLE_EDITOR:
            sample_editor_paste_clicked();
            gui_xm_set_modified(1);
            break;
        }
        break;
    }
}

void menubar_handle_edit_menu(gpointer a)
{
    if (GUI_EDITING) {
        Tracker* t = tracker;

        switch (GPOINTER_TO_INT(a)) {
        case 0:
            track_editor_cmd_mvalue(t, TRUE); /* increment CMD value */
            break;
        case 1:
            track_editor_cmd_mvalue(t, FALSE); /* decrement CMD value */
            break;
        case 2:
            transposition_transpose_selection(t, +1);
            break;
        case 3:
            transposition_transpose_selection(t, -1);
            break;
        case 4:
            transposition_transpose_selection(t, +12);
            break;
        case 5:
            transposition_transpose_selection(t, -12);
            break;
        default:
            break;
        }
    }
}

void menubar_settings_tracker_next_font(void)
{
    trackersettings_cycle_font_forward(TRACKERSETTINGS(trackersettings));
}

void menubar_settings_tracker_prev_font(void)
{
    trackersettings_cycle_font_backward(TRACKERSETTINGS(trackersettings));
}

void menubar_toggle_perm_wrapper(gpointer all)
{
    track_editor_toggle_permanentness(tracker, GPOINTER_TO_INT(all));
}

void menubar_mute_toggle(void)
{
    scope_group_toggle_channel(scopegroup, tracker->cursor_ch);
}

void menubar_solo_channel(void)
{
    scope_group_solo_channel(scopegroup, tracker->cursor_ch);
}

void menubar_all_channels_on(void)
{
    scope_group_all_channels_on(scopegroup);
}

void menubar_init_prefs()
{
    GtkWidget* display_scopes = gui_get_widget(gui_builder, "settings_display_scopes", XML_FILE);
    GtkWidget* backing_store = gui_get_widget(gui_builder, "settings_tracker_flicker_free", XML_FILE);
#if !defined(DRIVER_ALSA)
    GtkWidget* settings_midi = gui_get_widget(gui_builder, "settings_midi", XML_FILE);
#endif
#if USE_SNDFILE == 0 && !defined(AUDIOFILE_VERSION)
    GtkWidget* savewav = gui_get_widget(gui_builder, "file_save_wav", XML_FILE);
#endif
    GtkWidget* disable_splash = gui_get_widget(gui_builder, "settings_disable_splash", XML_FILE);
    GtkWidget* save_onexit = gui_get_widget(gui_builder, "settings_save_on_exit", XML_FILE);

    mark_mode = gui_get_widget(gui_builder, "edit_selection_mark_mode", XML_FILE);

    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(display_scopes), gui_settings.gui_display_scopes);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(backing_store), gui_settings.gui_use_backing_store);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(disable_splash), gui_settings.gui_disable_splash);
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(save_onexit), gui_settings.save_settings_on_exit);

#if USE_SNDFILE == 0 && !defined(AUDIOFILE_VERSION)
    gtk_widget_set_sensitive(savewav, FALSE);
#endif
#if !defined(DRIVER_ALSA)
    gtk_widget_set_sensitive(settings_midi, FALSE);
#endif
}

void menubar_block_mode_set(const gboolean state, const gboolean ignore)
{
    mark_mode_toggle_ignore = ignore;
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mark_mode), state);
    mark_mode_toggle_ignore = FALSE;
}
